package betterwithmods.common.registry.bulk.recipes;

import betterwithmods.common.BWOreDictionary;
import betterwithmods.common.registry.OreStack;
import betterwithmods.util.InvUtils;
import com.google.common.collect.Lists;
import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.NonNullList;
import net.minecraft.world.World;
import net.minecraftforge.items.ItemStackHandler;
import net.minecraftforge.oredict.OreDictionary;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;

public class BulkRecipe implements Comparable<BulkRecipe> {

    @Nonnull
    protected ItemStack output = ItemStack.field_190927_a;
    @Nonnull
    protected ItemStack secondary = ItemStack.field_190927_a;
    @Nonnull
    protected ArrayList<Object> inputs = new ArrayList<>();//Either ItemStack or OreStack
    @Nonnull
    protected ArrayList<List<ItemStack>> jeiInputs = new ArrayList<>();

    protected int priority = 0;

    protected BulkRecipe(ItemStack output, Object... inputs) {
        this(output, ItemStack.field_190927_a, inputs);
    }

    public BulkRecipe(@Nonnull ItemStack output, @Nonnull ItemStack secondaryOutput, Object... inputs) {
        this.output = output.func_77946_l();
        this.secondary = !secondaryOutput.func_190926_b() ? secondaryOutput.func_77946_l() : ItemStack.field_190927_a;
        int place = -1;
        ArrayList<Object> inputList = new ArrayList<>();
        for (Object in : inputs) {
            if (in instanceof Integer || in == null)
                continue;
            place++;
            if (in instanceof ItemStack) {
                inputList.add(((ItemStack) in).func_77946_l());
            } else if (in instanceof Item) {
                inputList.add(new ItemStack((Item) in, 1, OreDictionary.WILDCARD_VALUE));
            } else if (in instanceof Block) {
                inputList.add(new ItemStack((Block) in, 1, OreDictionary.WILDCARD_VALUE));
            } else if (in instanceof String) {
                int count = place + 1 < inputs.length && inputs[place + 1] instanceof Integer ? (int) inputs[place + 1] : 1;
                OreStack stack = new OreStack((String) in, count);
                if (!stack.isEmpty())
                    inputList.add(stack);
            } else if (in instanceof OreStack) {
                inputList.add(((OreStack) in).copy());
            } else {
                StringBuilder ret = new StringBuilder("Invalid " + this.getClass().getSimpleName() + " recipe: ");
                for (Object tmp : inputList)
                    ret.append(tmp).append(", ");
                ret.append("Output: ").append(output);
                if (!secondaryOutput.func_190926_b())
                    ret.append(", Secondary: ").append(secondaryOutput);
                throw new RuntimeException(ret.toString());
            }
        }
        condenseInputs(inputList);
    }

    public NonNullList<ItemStack> onCraft(World world, TileEntity tile, ItemStackHandler inv) {
        NonNullList<ItemStack> list = NonNullList.func_191197_a(2, ItemStack.field_190927_a);
        list.set(0, getOutput());
        list.set(1, getSecondary());
        this.consumeInvIngredients(inv);
        return list;
    }

    private void condenseInputs(ArrayList<Object> list) {
        for (Object obj : list) {
            int contain = BWOreDictionary.listContains(obj, inputs);
            if (contain > -1) {
                if (obj instanceof ItemStack) {
                    ((ItemStack) inputs.get(contain)).func_190917_f(((ItemStack) obj).func_190916_E());
                } else if (obj instanceof OreStack) {
                    ((OreStack) inputs.get(contain)).addToStack(((OreStack) obj).getStackSize());
                }
            } else {
                inputs.add(obj);
            }
        }
        for (Object obj : inputs) {
            List<ItemStack> in = new ArrayList<>();
            if (obj instanceof ItemStack)
                in.add((ItemStack) obj);
            else if (obj instanceof OreStack) {
                in.addAll(((OreStack) obj).getItems());
            }
            jeiInputs.add(in);
        }
    }


    public List<ItemStack> getOutputs() {
        return Lists.newArrayList(this.output.func_77946_l(), this.secondary.func_77946_l());
    }

    @Nonnull
    public ItemStack getOutput() {
        return this.output;
    }

    @Nonnull
    public ItemStack getSecondary() {
        return this.secondary;
    }

    @Nonnull
    public ArrayList<List<ItemStack>> getInputs() {
        return this.jeiInputs;
    }

    public ArrayList<Object> getRecipeInput() {
        return this.inputs;
    }

    public boolean matches(ItemStack output) {
        return this.output.func_77969_a(output);
    }

    public boolean matches(ItemStack output, ItemStack secondary) {
        return this.matches(output) && (secondary.func_190926_b() || this.secondary.func_77969_a(secondary));
    }

    public boolean matches(ItemStackHandler inv) {
        ArrayList<Object> required = new ArrayList<>(inputs);

        if (required.size() > 0) {
            for (Object obj : required) {
                if (obj instanceof ItemStack) {
                    ItemStack stack = (ItemStack) obj;
                    if (!stack.func_190926_b()) {
                        if (InvUtils.countItemStacksInInventory(inv, stack) < stack.func_190916_E())
                            return false;
                    }
                } else if (obj instanceof OreStack) {
                    OreStack stack = (OreStack) obj;
                    if (InvUtils.countOresInInventory(inv, stack.getItems()) < stack.getStackSize())
                        return false;
                }
            }
            return true;
        }
        return false;
    }

    public boolean consumeInvIngredients(ItemStackHandler inv) {
        boolean success = true;
        ArrayList<Object> required = inputs;

        if (required != null && required.size() > 0) {
            for (Object obj : required) {
                if (obj instanceof ItemStack) {
                    ItemStack stack = (ItemStack) obj;

                    if (!stack.func_190926_b()) {
                        if (!InvUtils.consumeItemsInInventory(inv, stack, stack.func_190916_E(), false))
                            success = false;
                    }
                } else if (obj instanceof OreStack) {
                    OreStack stack = (OreStack) obj;

                    if (stack != null) {
                        if (!InvUtils.consumeOresInInventory(inv, stack.getItems(), stack.getStackSize()))
                            success = false;
                    }
                }
            }
        }
        return success;
    }

    public boolean isEmpty() {
        return this.inputs.isEmpty() || getOutputs().isEmpty();
    }

    @Override
    public String toString() {
        return String.format("%s: %s -> %s,%s", getClass().getSimpleName(), this.inputs, this.output, this.secondary);
    }

    /**
     * Recipes with higher priority will be crafted first.
     * @return sorting priority for Comparable
     */
    public int getPriority() {
        return priority;
    }

    public void setPriority(int priority) {
        this.priority = priority;
    }

    @Override
    public int compareTo(BulkRecipe other) {
        return this.getPriority() == other.getPriority() ? 0 : this.getPriority() > other.getPriority() ? -1 : 1;
    }



}
